import { Module } from 'vuex'
import { RootState } from '@/store/store'
import { getStoreBuilder } from 'vuex-typex'
import { Subscription } from '@/models'

export const namespace = 'feedbackSearchOptions'

export interface FeedbackSearchOptionsState {
  showFinal: boolean
  searchOtherUserComments: boolean
  caseSensitive: boolean
  useRegex: boolean
  filterByTutors: string[]
  filterByStage: string
}

function initialState (): FeedbackSearchOptionsState {
  return {
    showFinal: false,
    searchOtherUserComments: false,
    caseSensitive: false,
    useRegex: false,
    filterByTutors: [],
    filterByStage: ''
  }
}

const mb = getStoreBuilder<RootState>().module('FeedbackSearchOptions', initialState())

const mapStageDisplayToApiString: {[index: string]: string} = {
  'Initial feedback': Subscription.FeedbackStageEnum.Creation,
  'Validation': Subscription.FeedbackStageEnum.Validation
}

const stateGetter = mb.state()

const stageFilterStringGetter = mb.read(function stageFilterString (state: FeedbackSearchOptionsState) {
  return mapStageDisplayToApiString[state.filterByStage] || 'all'
})

function SET_SHOW_FINAL (state: FeedbackSearchOptionsState, val: boolean) {
  state.showFinal = val
}
function SET_SEARCH_OTHER_USER_COMMENTS (state: FeedbackSearchOptionsState, val: boolean) {
  state.searchOtherUserComments = val
}
function SET_CASE_SENSITIVE (state: FeedbackSearchOptionsState, val: boolean) {
  state.caseSensitive = val
}
function SET_USE_REGEX (state: FeedbackSearchOptionsState, val: boolean) {
  state.useRegex = val
}
function SET_FILTER_BY_TUTORS (state: FeedbackSearchOptionsState, val: string[]) {
  state.filterByTutors = val
}
function SET_FILTER_BY_STAGE (state: FeedbackSearchOptionsState, val: string) {
  state.filterByStage = val
}

export const FeedbackSearchOptions = {
  get state () { return stateGetter() },
  get stageFilterString () { return stageFilterStringGetter() },

  SET_SHOW_FINAL: mb.commit(SET_SHOW_FINAL),
  SET_SEARCH_OTHER_USER_COMMENTS: mb.commit(SET_SEARCH_OTHER_USER_COMMENTS),
  SET_CASE_SENSITIVE: mb.commit(SET_CASE_SENSITIVE),
  SET_USE_REGEX: mb.commit(SET_USE_REGEX),
  SET_FILTER_BY_TUTORS: mb.commit(SET_FILTER_BY_TUTORS),
  SET_FILTER_BY_STAGE: mb.commit(SET_FILTER_BY_STAGE)
}
