import Vuex from 'vuex'
import { mount, createLocalVue } from '@vue/test-utils'
import AutoLogout from '@/components/AutoLogout.vue'
import sinon from 'sinon'
import { Authentication } from '@/store/modules/authentication'
import { SET_LAST_INTERACTION } from '@/store/mutations'
import { lastInteraction } from '@/store/plugins/lastInteractionPlugin'
import chai from 'chai'
chai.should()

let localVue = createLocalVue()
localVue.use(Vuex)

describe('Auto Logout Unit Tests', () => {
  let store: any = null
  let consoleTemp = {
    warn: console.warn,
    error: console.error
  }

  before(function () {
    console.warn = function () {}
    console.error = function () {}
  })

  after(function () {
    console.warn = consoleTemp.warn
    console.error = consoleTemp.error
  })

  beforeEach(() => {
    store = new Vuex.Store({
      plugins: [lastInteraction]
    })
  })

  afterEach(() => {
    sinon.restore()
  })

  it('should be hidden by default', () => {
    let wrapper = mount(AutoLogout, { localVue: localVue, store })
    wrapper.vm.$data.logoutDialog.should.equal(false)
    wrapper.html().should.contain('<div class="v-dialog v-dialog--persistent" style="max-width: 30%; display: none;">')
  })
  it('should be visible when logoutDialog is set to true', () => {
    let store = new Vuex.Store({})
    let wrapper = mount(AutoLogout, { localVue: localVue, store })
    wrapper.vm.$data.logoutDialog = true
    wrapper.html().should.contain('<div class="v-dialog v-dialog--active v-dialog--persistent" style="max-width: 30%;">')
  })
  it('should get a refresh token from the server when user clicks button', () => {
    let spy = sinon.spy()
    sinon.replace(Authentication, 'refreshJWT', spy)
    let store = new Vuex.Store({})
    let wrapper = mount(AutoLogout, { localVue: localVue, store })
    wrapper.find('#continue-btn').trigger('click')
    spy.called.should.equal(true)
  })
  it('should automatically update jwt when it is older than 20% of its lifetime', () => {
    // replace lastTokenRefreshTry and jwtTimeDelta from the store to emulate wanted behaviour
    let wrapper = mount(AutoLogout, { localVue: localVue, store })
    let spy = sinon.spy()
    sinon.replaceGetter(Authentication.state, 'jwtTimeDelta', () => 10 * 1e3)
    sinon.replaceGetter(Authentication.state, 'lastTokenRefreshTry', () => Date.now() - 2.1 * 1e3)
    sinon.replace(Authentication, 'refreshJWT', spy)
  })
})
